# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.

from .dyval_dataset import DyValDataset
from .dyval_utils import *


DYVAL_DATASETS = [
    "arithmetic",
    "linear_equation",
    "bool_logic",
    "deductive_logic",
    "abductive_logic",
    "reachability",
    "max_sum_path"
]